/*
 *  st_callbacks.c 
 *
 *  Callback-Definitionen fuer stopwatch.fd/stopwatch.c
 *
 *  (C) 10/1996 Tilman Kuepper (tilman@tilli.tng.oche.de)
 */


#include "forms.h"
#include "stopwatch.h"
#include "browser.h"
#include <time.h>
#include <stdio.h>


/* RCS-Infos */
/*
  static char RCSdat[]="$Date: 1996/10/16 16:42:32 $";
  static char RCSrev[]="$Revision: 2.5 $"; 
  static char RCSsta[]="$State: Rel $";
  */


/*
 *  globale Variablen
 */
static time_t diff=0; /* Differenz zwischen Systemzeit und angezeigter Zeit */
static time_t anz=0; /* angezeigte Zeit */
static time_t old_anz=-1; /* vorherige angezeigte Zeit */
static int run=0; /* 1: Uhr laeuft; 0: Uhr steht */
static FL_OBJECT *t_time=NULL; /* Textfeld mit Zeit */


/*
 *  Callbacks
 */
void cb_timer(FL_OBJECT *ob, long data)
{
    char labelstring[100]; /* String fuer Anzeige */
    
    old_anz=anz; /* alten Sekundenwert merken */
    if(run==1) /* neue Zeit nur berechnen, falls Uhr laeuft */
    {
        anz=time(NULL)-diff;
    }
    
    /* Nur anzeigen, falls neuer Wert vorhanden */
    if(anz!=old_anz)
    {
        sprintf(labelstring,"%02li : %02li : %02li",
		anz/3600,anz/60-((long)(anz/3600))*60,anz%60);
        fl_set_object_label(t_time,labelstring);     
    }
    
    /* Timer neu setzen */
    fl_set_timer(ob,0.2);

    return;
}


void cb_stop(FL_OBJECT *ob, long data)
{
    FD_f_stopwatch *fds=ob->u_vdata; /* Form ermitteln */
    
    /* STOP-Knopf gedrueckt */
    run=0;
    
    /* STOP-Knopf deaktivieren, START-Knopf aktivieren */
    fl_deactivate_object(ob);
    fl_set_object_lcol(ob,FL_INACTIVE_COL);
    fl_activate_object(fds->b_start);
    fl_set_object_lcol(fds->b_start,FL_BLACK);
    
    return;
}


void cb_reset(FL_OBJECT *ob, long data)
{
    char labelstring[100]; /* String fuer Anzeige */
    
    /* RESET-Knopf gedrueckt */
    anz=0;
    diff=time(NULL);
    
    /* Anzeige ruecksetzen */
    if(t_time!=NULL)
    {
        sprintf(labelstring,"%02i : %02i : %02i",0,0,0);
        fl_set_object_label(t_time,labelstring);     
    }
    
    return;
}


void cb_quit(FL_OBJECT *ob, long data)
{
    /* QUIT-Knopf gedrueckt */  
    fl_finish();
    exit(0);
}


void cb_start(FL_OBJECT *ob, long data)
{
    /* START-Knopf gedrueckt */
    FD_f_stopwatch *fds=ob->u_vdata; /* Form ermitteln */
    
    t_time=fds->t_time; /* Textfeld ermitteln und in glob. Variable merken */
    
    /* START-Knopf deaktivieren, STOP-Knopf aktivieren */
    fl_deactivate_object(ob);
    fl_set_object_lcol(ob,FL_INACTIVE_COL);
    fl_activate_object(fds->b_stop);
    fl_set_object_lcol(fds->b_stop,FL_BLACK);
    
    diff=time(NULL)-anz; /* Differenz fuer Zeitberechnung holen */
    run=1; /* Uhr laeuft! */
    
    /* Timer setzen */
    fl_set_timer(fds->my_timer,0.2);

    return;
}


void cb_list(FL_OBJECT *ob, long data)
{
    /* LIST-Knopf gedrueckt */
    char labelstring[100]; /* String fuer Eintrag im Browser */
    FD_f_browser *fdb=ob->u_vdata; /* Browser-Form ermitteln */
    
    fl_show_form(fdb->f_browser,FL_PLACE_FREE,FL_FULLBORDER,"list");
    sprintf(labelstring,"%02li : %02li : %02li",
	    anz/3600,anz/60-((long)(anz/3600))*60,anz%60);
    fl_addto_browser(fdb->br_list,labelstring);
    
    /* PRINT-Knopf im Browser aktivieren */
    fl_activate_object(fdb->b_print);
    fl_set_object_lcol(fdb->b_print,FL_BLACK);
}



